﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace extensionMethodsExercise
{
    public static class ExtensionMethods
    {
        public static String Concat<T>(this IEnumerable<T> elements, String separator)
        {
            return String.Join(", ", elements);
        }

        public static void Print(this Object obj)
        {
            //Console.WriteLine(obj);
            obj.Act(o => Console.WriteLine(obj));
        }

        // extra
        public static void Act(this Object obj, Action<Object> action)
        {
            action(obj);
        }

        public static void Print<T>(this IEnumerable<T> elements, String separator)
        {
            String str = String.Join(separator, elements);
            Console.WriteLine(str);
        }

        public static IEnumerable<T> Sort<T>(this IEnumerable<T> elements) where T : IComparable<T>
        {
            List<T> list = elements.ToList();
            list.Sort();
            return list;
        }

        public static IEnumerable<T> Sort<T>(this IEnumerable<T> elements, Comparison<T> comp)
        {
            List<T> list = elements.ToList();
            list.Sort(comp);
            return list;
        }

        private static readonly Random random = new Random();
        public static IEnumerable<int> GetRandom(int howmany, int max = Int32.MaxValue, int min = 0)
        {
            for (int i = 0; i < howmany; i++)
            {
                yield return random.Next(min, max);
            }
        }

        public static IEnumerable<T> Unique<T>(this IEnumerable<T> elements)
        {
            Boolean first = true;
            T previousElement = default(T);
            foreach (T element in elements)
            {
                if (first)
                {
                    first = false;
                    yield return element;
                }
                else
                {
                    if (!element.Equals(previousElement))
                    {
                        yield return element;
                    }
                }
                previousElement = element;
            }
        }

        public static IEnumerable<T> Unique<T>(this IEnumerable<T> elements, Func<T, T, bool> tester)
        {
            Boolean first = true;
            T previousElement = default(T);
            foreach (T element in elements)
            {
                if (first)
                {
                    first = false;
                    yield return element;
                }
                else
                {
                    if (tester(previousElement, element))
                    {
                        yield return element;
                    }
                }
                previousElement = element;
            }
        }

        public static IEnumerable<IEnumerable<T>> Split<T>(this IEnumerable<T> elements, int chunkSize)
        {
            Console.WriteLine("Split: " + chunkSize);
            if (chunkSize <= 0) throw new ArgumentOutOfRangeException("chunkSize");
            int number = 0;
            IList<T> list = new List<T>();
            foreach (T element in elements)
            {
                if (number == chunkSize)
                {
                    yield return list;
                    number = 0;
                    list = new List<T>();
                }
                list.Add(element);
                number++;
            }
            yield return list;
        }
    }
}
