﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.Remoting.Messaging;
using extensionMethodsExercise;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace extensionMethodsExerciseTests1
{
    [TestClass]
    public class HelpersTests
    {
        [TestMethod]
        public void ConcatTest()
        {
            IList<int> numbers = new[] { 4, 3, 2, 7, 2 };
            String result = numbers.Concat(", ");
            Assert.AreEqual("4, 3, 2, 7, 2", result);
        }

        [TestMethod]
        public void SortTest()
        {
            IList<int> numbers = new[] { 4, 3, 2, 7, 2 };
            IEnumerable<int> sorted = numbers.Sort();
            IList<int> expected = new[] { 2, 2, 3, 4, 7 };
            Assert.IsTrue(expected.SequenceEqual(sorted));
        }

        [TestMethod]
        public void SortTest1()
        {
            IList<int> numbers = new[] { 4, 3, 2, 7, 2 };
            IEnumerable<int> sorted2 = numbers.Sort((i, j) => j - i);
            IList<int> expected2 = new[] { 7, 4, 3, 2, 2 };
            Assert.IsTrue(expected2.SequenceEqual(sorted2));
        }

        [TestMethod]
        public void GetRandomTest()
        {
            IEnumerable<int> numbers = ExtensionMethods.GetRandom(10, 20, 8);
            Assert.AreEqual(10, numbers.Count());
            Assert.IsTrue(8 <= numbers.Min());
            Assert.IsTrue(numbers.Max() < 20);

            try
            {
                numbers = ExtensionMethods.GetRandom(10, 2, 5);
                numbers.ToList();
                Assert.Fail();
            }
            catch (ArgumentOutOfRangeException) { }

        }

        [TestMethod]
        public void UniqueTest()
        {
            IList<int> numbers = new[] { 4, 3, 2, 7, 2 };
            IEnumerable<int> result = numbers.Sort().Unique();
            IList<int> expected = new[] { 2, 3, 4, 7, };
            Assert.IsTrue(expected.SequenceEqual(result));
        }

        [TestMethod]
        public void UniqueTest1()
        {
            IList<int> numbers = new[] { 4, 3, 2, 7, 2 };
            IEnumerable<int> result = numbers.Unique((i, j) => i < j);
            IList<int> expected = new[] { 4, 7 };
            Assert.IsTrue(expected.SequenceEqual(result));
        }

        [TestMethod]
        public void SplitTest()
        {
            IList<int> numbers = new[] { 1, 4, 5, 3, 2, 6 };
            IEnumerable<IEnumerable<int>> parts = numbers.Split(4);
            Assert.AreEqual(2, parts.Count());

            parts = numbers.Split(3);
            Assert.AreEqual(2, parts.Count());

            parts = numbers.Split(10);
            Assert.AreEqual(1, parts.Count());

            parts = numbers.Split(1);
            Assert.AreEqual(6, parts.Count());

            try
            {
                parts = numbers.Split(-6);
                parts.ToList(); // iterator is lazy, must be activated to produce the exception
                Assert.Fail();
            }
            catch (ArgumentOutOfRangeException) { }
        }
    }
}
